--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
--  Page 38
--  Intro to Pseudo Cursors - Using the "Presence of Rows"
--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
GO
--===== Example 1 - Returns all rows and columns, which means it returns DATA from the table.
 SELECT *
   FROM sys.all_columns
;
--===== Example 2 - Returns one "1" for each row in the table. Used NO data from the table.
     --             It only used the "Presence of Rows".  This is a Pseudo Cursor!
 SELECT N = 1
   FROM sys.all_columns
;
--===== Example 3 - Fast way to count. Still a Pseudo Cursor that only used the "Presence of Rows"
 SELECT N = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM sys.all_columns
;
--===== Example 4 - Trying to count to a million but... not enough rows.
 SELECT TOP (1000000)
        N = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM sys.all_columns
;
--===== Example 5 - CROSS JOIN is a row "Multiplier". 10,000 * 10,000 = 100,000,000.
     --             TOP() sets what is known as a "Row Goal". 
     --             Do NOT use WHERE as a Row Goal because it would have to calculate all the values first!
     --             This takes more than 6 seconds because of "Display time"
 SELECT TOP (1000000)
        N = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM      sys.all_columns ac1
  CROSS JOIN sys.all_columns ac2
;
--===== Example 6 - To test without "Display Time", dump the output to a "Throw Away" variable.
     --             You could also change the SSMS setting to "Discard results after execution"
     --             but that's a pain if you want to see some results but not others. 
     --             This took only ~82 ms.  
    SET STATISTICS TIME,IO ON
;
DECLARE @BitBucket INT;
 SELECT TOP (1000000)
        @BitBucket = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM      sys.all_columns ac1
  CROSS JOIN sys.all_columns ac2
;
    SET STATISTICS TIME,IO OFF
;
--===== Example 7 - Using SELECT/INTO to build a table and then add a clustered index.
   DROP TABLE IF EXISTS #MyHead;
    SET STATISTICS TIME,IO ON
;
 SELECT TOP (1000000)
        N = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   INTO #MyHead
   FROM      sys.all_columns ac1
  CROSS JOIN sys.all_columns ac2
;
    SET STATISTICS TIME,IO OFF;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 CREATE UNIQUE CLUSTERED INDEX PK_#MyHead ON #MyHead (N)
;
    SET STATISTICS TIME,IO OFF
;
/*
 SELECT *
   FROM #MyHead
  ORDER BY N
;
*/
--===== Example 8 - Build the table and the Clustered Index, then populate. 
     --             This will not be "Minimally Logged" because it's not setup right.
     --             Still, it does get rid of the time to build the Clustered Index.
   DROP TABLE IF EXISTS #MyHead
;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 CREATE TABLE #MyHead
        (N INT NOT NULL)
;
 CREATE UNIQUE CLUSTERED INDEX PK_#MyHead ON #MyHead (N)
;
    SET STATISTICS TIME,IO OFF;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 INSERT INTO #MyHead 
 SELECT TOP (1000000)
        N = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM      sys.all_columns ac1
  CROSS JOIN sys.all_columns ac2
;
    SET STATISTICS TIME,IO OFF
;
  PRINT REPLICATE('-',119)
;

/*
 SELECT *
   FROM #MyHead
  ORDER BY N
;
*/
--===== Example 9 - Same as example 8 but "Minimally Logged" because of TABLOCK
     --             AND ORDER BY in ROW_NUMBER(),
   DROP TABLE IF EXISTS #MyHead
;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 CREATE TABLE #MyHead
        (N INT NOT NULL)
;
 CREATE UNIQUE CLUSTERED INDEX PK_#MyHead ON #MyHead (N)
;
    SET STATISTICS TIME,IO OFF;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 INSERT INTO #MyHead WITH (TABLOCK)
 SELECT TOP (1000000)
        N = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM      sys.all_columns ac1
  CROSS JOIN sys.all_columns ac2
  ORDER BY N -- This ORDER BY doesn't do a thing but doesn't hurt either. Best to be safe.
;
    SET STATISTICS TIME,IO OFF
;
  PRINT REPLICATE('-',119)
;
/*
 SELECT *
   FROM #MyHead
  ORDER BY N
;
*/
--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
--  Page 39
--  Intro to READLESS Pseudo Cursors
--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
GO
--===== Example 1 - "Table Value Constructor" (VALUES) returns 16 "1"s with NO READs.
    SET STATISTICS TIME, IO ON
;
 SELECT 1 FROM (VALUES (1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1))E0(N)
;
    SET STATISTICS TIME, IO OFF
;
--===== Example 2 - 7 CROSS JOINS of 8 "copies" of the "Table Valued Constructor to  
     --           - provide up a "Presence of Rows" up to 16^8 or 4,294,967,296.
     --             Again, using TOP() as the "Row Goal" and, again, NO READs.
    SET STATISTICS TIME, IO ON
;
   WITH
  E1(N) AS (SELECT 1 FROM (VALUES (1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1))E0(N))
 SELECT TOP (1000000) ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM E1 a,E1 b,E1 c,E1 d,E1 e,E1 f,E1 g,E1 h
;
    SET STATISTICS TIME, IO OFF
;
--===== Example 3 - Same as example 2 but dumping to a "Throw Away Variable"
     --             Say it with me!!! NASTY FAST!!!
DECLARE @BitBucket INT
;
    SET STATISTICS TIME,IO ON
;
   WITH
  E1(N) AS (SELECT 1 FROM (VALUES (1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1),(1))E0(N))
 SELECT TOP (1000000) @BitBucket = ROW_NUMBER() OVER (ORDER BY (SELECT NULL))
   FROM E1 a,E1 b,E1 c,E1 d,E1 e,E1 f,E1 g, E1 h
;
    SET STATISTICS TIME,IO OFF
;
--===== Example 4 - This uses the fnTally function from the ZIP file to do the same thing
     --             but it has some built in protection against bad execution plans during
     --             parallelism that cause "Late Row Goals" that cause all 4,294,967,296
     --             rows to be generated internally and then be "filtered" to just 1,000,000.
     --             Again, display time causes a perceived duration issue.
    SET STATISTICS TIME,IO ON
;
 SELECT *
   FROM dbo.fnTally(1,1000000)
;
    SET STATISTICS TIME,IO OFF
;
--===== Example 5 - Same as example 4 but directed to a "Throw Away Variable".
     --             Again, NASTY FAST!!!  Not real useful EXCEPT for measuring methods.
DECLARE @BitBucket INT
;
    SET STATISTICS TIME,IO ON
;
 SELECT @BitBucket = N
   FROM dbo.fnTally(1,1000000)
;
    SET STATISTICS TIME,IO OFF
;
--===== Example 6 - Again, same as example 4 but directed to a Temp Table.
     --             Still, NASTY FAST!!!
   DROP TABLE IF EXISTS #MyHead
;
    SET STATISTICS TIME,IO ON
;
 SELECT N
   INTO #MyHead
   FROM dbo.fnTally(1,1000000)
;
    SET STATISTICS TIME,IO OFF
;
--===== Example 7 - Again, same as example 4 but directed to a Temp Table and then.
     --             building a Clustered Index.  Still, NASTY FAST!!!
   DROP TABLE IF EXISTS #MyHead;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 SELECT N
   INTO #MyHead
   FROM dbo.fnTally(1,1000000)
;
    SET STATISTICS TIME,IO OFF;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 CREATE UNIQUE CLUSTERED INDEX PK_#MyHead ON #MyHead (N)
;
    SET STATISTICS TIME,IO OFF
;
/*
 SELECT *
   FROM #MyHead
  ORDER BY N
;
*/
--===== Example 8 - Same as 7 but build the table and the Clustered Index, then populate. 
     --             This will not be "Minimally Logged" because it's not setup right.
   DROP TABLE IF EXISTS #MyHead;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;

 CREATE TABLE #MyHead
        (N INT NOT NULL)
;
 CREATE UNIQUE CLUSTERED INDEX PK_#MyHead ON #MyHead (N)
;
    SET STATISTICS TIME,IO OFF;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 INSERT INTO #MyHead 
 SELECT N 
   FROM dbo.fnTally(1,1000000)
;
    SET STATISTICS TIME,IO OFF
;
--===== Example 9 - Same as 8 but build the table and the Clustered Index, then populate. 
     --             This will be "Minimally Logged" because it's of TABLOCK and ORDER BY
     --             in the fnTally function.
   DROP TABLE IF EXISTS #MyHead;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 CREATE TABLE #MyHead
        (N INT NOT NULL)
;
 CREATE UNIQUE CLUSTERED INDEX PK_#MyHead ON #MyHead (N)
;
    SET STATISTICS TIME,IO OFF;
  PRINT REPLICATE('-',119);
    SET STATISTICS TIME,IO ON
;
 INSERT INTO #MyHead WITH (TABLOCK)
 SELECT N
   FROM dbo.fnTally(1,1000000)
  ORDER BY N       -- This time, this can cause a bit of a savings in CPU and Duration
 OPTION (MAXDOP 1) -- This can sometimes help a lot on "intermediate large" numbers of rows.
;
    SET STATISTICS TIME,IO OFF
;
checkpoint;
DBCC FREEPROCCACHE;
DBCC dropcleanbuffers;


