--=====================================================================================================================
--      Create a 30 million row test table and populate it on-the-fly to simulate phone calls taken.
--=====================================================================================================================
--===== Conditionally drop the test table to make reruns easier
     IF OBJECT_ID('dbo.BigCallTest','U') IS NOT NULL
        DROP TABLE dbo.BigCallTest
;
GO
--===== Create and populate a 30,000,000 row test table.
     -- "CallTestID"   has a domain of 1 to 30,000,000 unique numbers
     -- "CallDateTime" has a domain of >=01/01/2019 and <01/01/2021
     -- "CsrTestID"    has a domain of 1 to 100
     -- "CallCode"     has a domain of A-E and V-Z (25 different call codes) 
     -- "CallMinutes"  has a domain of 1 to 15 minutes
 SELECT  CallTestID     = IDENTITY(INT,1,1)
        ,CallDateTime   = RAND(CHECKSUM(NEWID())) * DATEDIFF(dd,'2019','2021') + CAST('2019' AS DATETIME)
        ,CsrTestID      = ABS(CHECKSUM(NEWID()) % 100) + 1
        ,CallCode       = CHAR(ABS(CHECKSUM(NEWID()) % (ASCII('E')-ASCII('A')+1)) +ASCII('A'))
                        + CHAR(ABS(CHECKSUM(NEWID()) % (ASCII('Z')-ASCII('V')+1)) +ASCII('V'))
        ,CallMinutes    = ABS(CHECKSUM(NEWID()) % 15) + 1 
   INTO dbo.BigCallTest
   FROM dbo.fnTally(1,30000000)
  ORDER BY CallDateTime 
  --OPTION (MAXDOP 1) --Slowed things way down for this 
;
--===== Create the standard Clustered Index (takes about 15 seconds)
  ALTER TABLE dbo.BigCallTest
    ADD CONSTRAINT PK_BigCallTest PRIMARY KEY CLUSTERED (CallTestID)
;
--===== Display some of the rows to see what we have.
 SELECT TOP 10000 * FROM dbo.bigcalltest ORDER BY CallDateTime
;
GO
--=====================================================================================================================
--      Create a 100 row table to simulate a CSR table, the people who take the calls.
--=====================================================================================================================
--===== Conditionally drop the test table to make reruns easier
     IF OBJECT_ID('dbo.BigCsrTest','U') IS NOT NULL
        DROP TABLE dbo.BigCsrTest
;
GO
--===== Create a CSR table that has 100 CsrTestIDs and Names
 SELECT  CsrTestID = ISNULL(N,0) --ISNULL makes this column NOT NULL
        ,CsrName   = 'CSR-'+RIGHT('000'+CAST(N AS VARCHAR(10)),3)
   INTO dbo.BigCsrTest
   FROM dbo.fnTally(1,100)
;
--===== Display what's in the table
 SELECT * FROM dbo.BigCsrTest
;
--===== Create the standard Clustered Index
  ALTER TABLE dbo.BigCsrTest
    ADD CONSTRAINT PK_BigCsrTest PRIMARY KEY CLUSTERED (CsrTestID)
;
GO
--=====================================================================================================================
--      Generate an "impossible" report.
--      1. Report number of calls and hours for previous 4 weeks by... (Except I tested with 8)
--      2. Each week for each CSR by day of week
--      3. Weekly totals and Average call duration for each CSR
--      4. Total by day across the desired weeks for each CSR
--      5. Totals and Average call duration for the desired weeks for each CSR 
--      6. Grand Totals for each week by day and total including Average call duration
--      7. Grand Totals for all weeks by day and total including Average call duration
--=====================================================================================================================
/*
CHECKPOINT;
DBCC FREEPROCCACHE;
DBCC DROPCLEANBUFFERS;
*/
SET STATISTICS TIME,IO ON;
--===== The Impossible Report
WITH
ctePreAgg AS
(--==== Work out the aggregates (Pre-Aggregation)
 SELECT  CsrTestID
        ,WeekDate  = DATEADD(dd, DATEDIFF(dd,-1,CallDateTime)/7*7,-1)    --First Sunday of the week for each row
        ,DOW       = DATEPART(dw,CallDateTime)                           --Sunday is 1, Monday is 2, etc
        ,CallHours = CAST(SUM(CallMinutes)/60.0 AS DECIMAL(9,1))         --60 minutes per hour
        ,CallCount = COUNT(*)                                            --Call Count per CSR per day
   FROM dbo.BigCallTest
  WHERE CallDateTime >= DATEADD(dd, DATEDIFF(dd,-1,GETDATE())/7*7-56, -1) --Sunday start of week 8 weeks ago
    AND CallDateTime <  DATEADD(dd, DATEDIFF(dd,-1,GETDATE())/7*7   , -1) --Sunday start of current week
  GROUP BY CsrTestID, DATEPART(dw,CallDateTime),DATEDIFF(dd,-1,CallDateTime)/7*7
)--==== Do the CROSS TAB display
 SELECT  csr.CsrTestID
        ,CsrName   = CASE
                     WHEN GROUPING(csr.CsrTestID) = 1 AND GROUPING(pa.WeekDate) = 0 
                     THEN CONVERT(CHAR(10),MAX(WeekDate),101)
                     WHEN GROUPING(csr.CsrTestID) = 1 AND GROUPING(pa.WeekDate) = 1 THEN ''
                     ELSE MAX(csr.CsrName)
                     END

        ,WeekDate  = CASE
                     WHEN GROUPING(csr.CsrTestID) = 0 AND GROUPING(pa.WeekDate) = 1 THEN 'Subtotal CSR'
                     WHEN GROUPING(csr.CsrTestID) = 1 AND GROUPING(pa.WeekDate) = 1 THEN 'Grand Total'
                     WHEN GROUPING(csr.CsrTestID) = 1 AND GROUPING(pa.WeekDate) = 0 THEN 'Subtotal Week'
                     ELSE CONVERT(CHAR(10),MAX(WeekDate),101)
                     END

        ,Qty_Sun   = SUM(CASE WHEN pa.DOW = 1 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Sun   = SUM(CASE WHEN pa.DOW = 1 THEN pa.CallHours ELSE 0 END)

        ,Qty_Mon   = SUM(CASE WHEN pa.DOW = 2 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Mon   = SUM(CASE WHEN pa.DOW = 2 THEN pa.CallHours ELSE 0 END)

        ,Qty_Tue   = SUM(CASE WHEN pa.DOW = 3 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Tue   = SUM(CASE WHEN pa.DOW = 3 THEN pa.CallHours ELSE 0 END)

        ,Qty_Wed   = SUM(CASE WHEN pa.DOW = 4 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Wed   = SUM(CASE WHEN pa.DOW = 4 THEN pa.CallHours ELSE 0 END)

        ,Qty_Thu   = SUM(CASE WHEN pa.DOW = 5 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Thu   = SUM(CASE WHEN pa.DOW = 5 THEN pa.CallHours ELSE 0 END)

        ,Qty_Fri   = SUM(CASE WHEN pa.DOW = 6 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Fri   = SUM(CASE WHEN pa.DOW = 6 THEN pa.CallHours ELSE 0 END)

        ,Qty_Sat   = SUM(CASE WHEN pa.DOW = 7 THEN pa.CallCount ELSE 0 END)
        ,Hrs_Sat   = SUM(CASE WHEN pa.DOW = 7 THEN pa.CallHours ELSE 0 END)

        ,Qty_Total = SUM(pa.CallCount)
        ,Hrs_Total = SUM(pa.CallHours)
        ,AvgDur    = CONVERT(CHAR(8),CAST(SUM(pa.CallHours)/SUM(pa.CallCount)/24.0 AS DATETIME),108)

   FROM ctePreAgg pa
   FULL JOIN dbo.BigCsrTest csr ON pa.CsrTestID = csr.CsrTestID
  GROUP BY csr.CsrTestID, pa.WeekDate WITH CUBE
  ORDER BY GROUPING(csr.CsrTestID), csr.CsrTestID, GROUPING(pa.WeekDate), pa.WeekDate
;
SET STATISTICS TIME,IO OFF;
GO

---------------------------------------------------------------------------------------------------------------
--===== Run this to see how big the various parts of the table are. Notice the very small index space.
   EXEC sp_spaceused 'dbo.BigCallTest'
;
--===== Run this to see how many rows are extracted from the large table to run the report
 SELECT COUNT(*)
   FROM dbo.BigCallTest
  WHERE CallDateTime >= DATEADD(dd, DATEDIFF(dd,-1,GETDATE())/7*7-56, -1) --Sunday start of previous week
    AND CallDateTime <  DATEADD(dd, DATEDIFF(dd,-1,GETDATE())/7*7  , -1) 
;
--===== Run this just to verify that there are 30 million rows in the table.
 SELECT COUNT(*)
   FROM dbo.BigCallTest
;
--===== Run this to create the index that the execution plan recommends and then run the report again.
     -- Notice that there's not a whole lot of diffence except by percentage of increase.
 CREATE NONCLUSTERED INDEX IX_BigCallTest_CallDateTime_Cover01
     ON dbo.BigCallTest (CallDateTime)
INCLUDE (CsrTestID, CallMinutes)
;
--===== Run this to see how big the various parts of the table are now. Notice the huge index space.
   EXEC sp_spaceused 'dbo.BigCallTest'
;
--===== This is just here if we need it if a question comes up.
   DROP INDEX IX_BigCallTest_CallDateTime_Cover01 ON dbo.BigCallTest
;
GO
