--=====================================================================================================================
--      Conditionally drop and recreate a View that will allow us to use NEWID() within a function so we can make
--      Random numbers in a function and create a function that will create constrained randomized CSV element rows.
--=====================================================================================================================
--===== Conditionally drop the objects in the code below to make reruns easier (Commented out for safety)
     --IF OBJECT_ID('dbo.iFunction'  ,'V' ) IS NOT NULL DROP VIEW     dbo.iFunction;
     --IF OBJECT_ID('dbo.CreateCsv8K','IF') IS NOT NULL DROP FUNCTION dbo.CreateCsv8K;
GO
 CREATE VIEW dbo.iFunction AS 
/**********************************************************************************************************************
 Purpose:
 This view is callable from UDF's which allows us to indirectly get a NEWID() within a function where we can't do such
 a thing directly in the function.  This view also solves the same problem for GETDATE().

 Usage:
 SELECT MyNewID FROM dbo.iFunction; --Returns a GUID
 SELECT MyDate  FROM dbo.iFunction; --Returns a Date


 Revision History:
 Rev 00 - 06 Jun 2004 - Jeff Moden - Initial creation
 Rev 01 - 06 Mar 2011 - Jeff Moden - Formalize code.  No logic changes.
**********************************************************************************************************************/
SELECT MyNewID = NEWID(),
       MyDate  = GETDATE();
GO 
 CREATE FUNCTION dbo.CreateCsv8K
/**********************************************************************************************************************
 Purpose:
 Create a CSV table result with a programmable number of rows, elements per row, minimum # of characters per element,
 and maximum characters per element.  The element size is random in nature constrained by the min and max characters
 per element.

 Usage:
--===== Syntax
 SELECT * FROM dbo.CreateCsv8K(@pNumberOfRows, @pNumberOfElementsPerRow, @pMinElementwidth, @pMaxElementWidth)
;
--===== Example Usage
     IF OBJECT_ID('dbo.Csv8K','U') IS NOT NULL DROP TABLE dbo.Csv8K;
 SELECT * INTO dbo.Csv8K FROM dbo.CreateCsv8K(100000, 110, 1, 7);
 SELECT * FROM dbo.Csv8K
;

 Dependencies:
 1. View: dbo.FunctionsForFunctions (Produces a NEWID() usable from within a UDF)

 Programmer's Notes:
 1. The randomness of the elements prevents the delimiters for showing up in the same position for each row so that
    SQL Server won't figure that out and cache the information making some splitting and other techniques seem faster
     than they really are.
 2. No validation or constraints have been place on the input parameters so use with caution.  This code can generate
    a lot of data in a couple of heart beats.

 Revision History:
 Rev 00 - 11 May 2007 - Jeff Moden - Initial creation - Only returned one row and wasn't programmable.
 Rev 01 - 26 Jul 2009 - Jeff Moden - Added programmable variables but would only go to 20 characters wide.
 Rev 02 - 06 Mar 2011 - Jeff Moden - Converted to iTVF, added minimum element width, and made it so elements can be
                                     virtually any size within 8k Bytes.
 Rev 03 - 14 Sep 2015 - Jeff Moden - Changed content of each element from a simple number to be much more complex
                                     and to include letters instead of just digits by using part of a NEWID().
 Rev 04 - 28 Sep 2015 - Jeff Moden - Added extra correlation between the "inside" and "outside" queries to prevent
                                     "Grooved Data".  Even  OPTION (QUERYTRACEON 9481) would no longer help thanks to
                                     "improvements in the optimizer".
**********************************************************************************************************************/
--===== Declare the I/0
        (
        @pNumberOfRows           INT,
        @pNumberOfElementsPerRow INT,
        @pMinElementwidth        INT,
        @pMaxElementWidth        INT
        )
RETURNS TABLE --Can't use WITH SCHEMABINDING because this references a system object.
     AS
 RETURN
--===== This creates and populates a test table on the fly containing a
     -- sequential column and a randomly generated CSV Parameter column.
 SELECT TOP (@pNumberOfRows) --Controls the number of rows in the test table
        ISNULL(ROW_NUMBER() OVER (ORDER BY(SELECT NULL)),0) AS RowNum,
        CSV =
        (--==== This creates each CSV
         SELECT CAST(
                    STUFF( --=== STUFF get's rid of the leading comma
                         ( --=== This builds CSV row with a leading comma
                          SELECT TOP (@pNumberOfElementsPerRow) --Controls the number of CSV elements in each row
                                 ','
                               + LEFT(--===== Limit the line length
                                      LEFT(--==== Create and limit the length of individual elements
                                           REPLICATE(
                                                      (SELECT RIGHT(MyNewID,10) FROM dbo.iFunction)
                                                      ,CEILING(@pMaxElementWidth/10.0)
                                                    )
                                          ,@pMaxElementWidth
                                          ),
                                      ABS(CHECKSUM((SELECT MyNewID FROM dbo.iFunction)))
                                      % (@pMaxElementWidth - @pMinElementwidth + 1) + @pMinElementwidth
                                     )
                            FROM      sys.All_Columns ac3       --Classic cross join pseudo-cursor
                           CROSS JOIN sys.All_Columns ac4       --can produce row sets up 16 million. (Much more nowadays)
                           WHERE ac3.Object_ID <> ac1.Object_ID --Without this line, all rows would be the same.
                             AND ac4.Object_ID <> ac2.Object_ID --Without this line, all rows would be the same.
                             FOR XML PATH('')
                         )
                    ,1,1,'') --Part of the STUFF to get rid of first comma.
                AS VARCHAR(8000))
        )
   FROM      sys.All_Columns ac1 --Classic cross join pseudo-cursor
  CROSS JOIN sys.All_Columns ac2 --can produce row sets up 16 million rows (Much more nowadays)
;
GO